@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import io
 import itertools
 import math
@@ -5,7 +7,7 @@ import operator
 import uuid
 import warnings
 from collections import defaultdict
-from collections.abc import Iterable, Iterator
+from collections.abc import Hashable, Iterable, Iterator, Mapping
 from functools import partial, reduce, wraps
 from random import Random
 from urllib.request import urlopen
@@ -35,16 +37,19 @@ from tlz import (
     valmap,
 )
 
-from .. import config
-from ..base import DaskMethodsMixin, dont_optimize, replace_name_in_key, tokenize
-from ..context import globalmethod
-from ..core import flatten, get_dependencies, istask, quote, reverse_dict
-from ..delayed import Delayed, unpack_collections
-from ..highlevelgraph import HighLevelGraph
-from ..multiprocessing import get as mpget
-from ..optimization import cull, fuse, inline
-from ..sizeof import sizeof
-from ..utils import (
+from dask import config
+from dask.bag import chunk
+from dask.bag.avro import to_avro
+from dask.base import DaskMethodsMixin, dont_optimize, replace_name_in_key, tokenize
+from dask.blockwise import blockwise
+from dask.context import globalmethod
+from dask.core import flatten, get_dependencies, istask, quote, reverse_dict
+from dask.delayed import Delayed, unpack_collections
+from dask.highlevelgraph import HighLevelGraph
+from dask.multiprocessing import get as mpget
+from dask.optimization import cull, fuse, inline
+from dask.sizeof import sizeof
+from dask.utils import (
     apply,
     digit,
     ensure_bytes,
@@ -58,8 +63,6 @@ from ..utils import (
     system_encoding,
     takes_multiple_arguments,
 )
-from . import chunk
-from .avro import to_avro
 
 no_default = "__no__default__"
 no_result = type(
@@ -458,7 +461,7 @@ class Bag(DaskMethodsMixin):
     30
     """
 
-    def __init__(self, dsk, name, npartitions):
+    def __init__(self, dsk: Mapping, name: str, npartitions: int):
         if not isinstance(dsk, HighLevelGraph):
             dsk = HighLevelGraph.from_collections(name, dsk, dependencies=[])
         self.dask = dsk
@@ -468,7 +471,7 @@ class Bag(DaskMethodsMixin):
     def __dask_graph__(self):
         return self.dask
 
-    def __dask_keys__(self):
+    def __dask_keys__(self) -> list[Hashable]:
         return [(self.name, i) for i in range(self.npartitions)]
 
     def __dask_layers__(self):
@@ -1474,7 +1477,7 @@ class Bag(DaskMethodsMixin):
         grouper,
         method=None,
         npartitions=None,
-        blocksize=2 ** 20,
+        blocksize=2**20,
         max_branch=None,
         shuffle=None,
     ):
@@ -1716,7 +1719,7 @@ def accumulate_part(binop, seq, initial, is_first=False):
     return res[1:], res[-1]
 
 
-def partition(grouper, sequence, npartitions, p, nelements=2 ** 20):
+def partition(grouper, sequence, npartitions, p, nelements=2**20):
     """Partition a bag along a grouper, store partitions on disk."""
     for block in partition_all(nelements, sequence):
         d = groupby(grouper, block)
@@ -2230,13 +2233,11 @@ def map_partitions(func, *args, **kwargs):
     single graph, and then computes everything at once, and in some cases
     may be more efficient.
     """
-    name = "{}-{}".format(
-        funcname(func), tokenize(func, "map-partitions", *args, **kwargs)
-    )
-    dependencies = []
-
+    name = kwargs.pop("token", None) or funcname(func)
+    name = "{}-{}".format(name, tokenize(func, "map-partitions", *args, **kwargs))
     bags = []
     args2 = []
+    dependencies = []
     for a in args:
         if isinstance(a, Bag):
             bags.append(a)
@@ -2278,7 +2279,9 @@ def map_partitions(func, *args, **kwargs):
             (zip, list(bag_kwargs), [(b.name, n) for b in bag_kwargs.values()]),
         )
 
-    if kwargs:
+    if bag_kwargs:
+        # Avoid using `blockwise` when a key-word
+        # argument is being used to refer to a collection.
         dsk = {
             (name, n): (
                 apply,
@@ -2289,7 +2292,32 @@ def map_partitions(func, *args, **kwargs):
             for n in range(npartitions)
         }
     else:
-        dsk = {(name, n): (func,) + tuple(build_args(n)) for n in range(npartitions)}
+        pairs = []
+        numblocks = {}
+        for arg in args2:
+            # TODO: Allow interaction with Frame/Array
+            # collections with proper partitioning
+            if isinstance(arg, Bag):
+                pairs.extend([arg.name, "i"])
+                numblocks[arg.name] = (arg.npartitions,)
+            else:
+                pairs.extend([arg, None])
+        if other_kwargs and isinstance(other_kwargs, tuple):
+            # `other_kwargs` is a nested subgraph,
+            # designed to generate the kwargs lazily.
+            # We need to convert this to a dictionary
+            # before passing to `blockwise`
+            other_kwargs = other_kwargs[0](other_kwargs[1][0](*other_kwargs[1][1:]))
+        dsk = blockwise(
+            func,
+            name,
+            "i",
+            *pairs,
+            numblocks=numblocks,
+            concatenate=True,
+            dependencies=dependencies,
+            **other_kwargs,
+        )
 
     # If all bags are the same type, use that type, otherwise fallback to Bag
     return_type = set(map(type, bags))
@@ -2309,7 +2337,7 @@ def _reduce(binop, sequence, initial=no_default):
 
 def make_group(k, stage):
     def h(x):
-        return x[0] // k ** stage % k
+        return x[0] // k**stage % k
 
     return h
 
@@ -2328,7 +2356,7 @@ def groupby_tasks(b, grouper, hash=hash, max_branch=32):
     splits = []
     joins = []
 
-    inputs = [tuple(digit(i, j, k) for j in range(stages)) for i in range(k ** stages)]
+    inputs = [tuple(digit(i, j, k) for j in range(stages)) for i in range(k**stages)]
 
     b2 = b.map(partial(chunk.groupby_tasks_group_hash, hash=hash, grouper=grouper))
 
@@ -2404,7 +2432,7 @@ def groupby_tasks(b, grouper, hash=hash, max_branch=32):
     return type(b)(graph, name, len(inputs))
 
 
-def groupby_disk(b, grouper, npartitions=None, blocksize=2 ** 20):
+def groupby_disk(b, grouper, npartitions=None, blocksize=2**20):
     if npartitions is None:
         npartitions = b.npartitions
     token = tokenize(b, grouper, npartitions, blocksize)
