@@ -24,6 +24,7 @@ from dask.dataframe.dispatch import (
     concat_dispatch,
     get_parallel_type,
     group_split_dispatch,
+    grouper_dispatch,
     hash_object_dispatch,
     is_categorical_dtype_dispatch,
     make_meta_dispatch,
@@ -535,6 +536,11 @@ def is_categorical_dtype_pandas(obj):
     return pd.api.types.is_categorical_dtype(obj)
 
 
+@grouper_dispatch.register((pd.DataFrame, pd.Series))
+def get_grouper_pandas(obj):
+    return pd.core.groupby.Grouper
+
+
 @percentile_lookup.register((pd.Series, pd.Index))
 def percentile(a, q, interpolation="linear"):
     return _percentile(a, q, interpolation)
