@@ -1,8 +1,10 @@
 import os
+from collections.abc import Iterable
 from functools import partial
 from math import ceil
 from operator import getitem
 from threading import Lock
+from typing import Optional, Union
 
 import numpy as np
 import pandas as pd
@@ -14,9 +16,13 @@ from dask.dataframe.core import (
     DataFrame,
     Index,
     Series,
+    _concat,
+    _emulate,
+    apply_and_enforce,
     has_parallel_type,
     new_dd_object,
 )
+from dask.dataframe.io.utils import DataFrameIOFunction
 from dask.dataframe.shuffle import set_partition
 from dask.dataframe.utils import (
     check_meta,
@@ -27,7 +33,7 @@ from dask.dataframe.utils import (
 from dask.delayed import delayed
 from dask.highlevelgraph import HighLevelGraph
 from dask.layers import DataFrameIOLayer
-from dask.utils import M, _deprecated
+from dask.utils import M, _deprecated, funcname, is_arraylike
 
 lock = Lock()
 
@@ -141,7 +147,13 @@ def from_array(x, chunksize=50000, columns=None, meta=None):
     return new_dd_object(dsk, name, meta, divisions)
 
 
-def from_pandas(data, npartitions=None, chunksize=None, sort=True, name=None):
+def from_pandas(
+    data: Union[pd.DataFrame, pd.Series],
+    npartitions: Optional[int] = None,
+    chunksize: Optional[int] = None,
+    sort: bool = True,
+    name: Optional[str] = None,
+) -> DataFrame:
     """
     Construct a Dask DataFrame from a Pandas DataFrame
 
@@ -212,24 +224,38 @@ def from_pandas(data, npartitions=None, chunksize=None, sort=True, name=None):
         raise NotImplementedError("Dask does not support MultiIndex Dataframes.")
 
     if not has_parallel_type(data):
-        raise TypeError("Input must be a pandas DataFrame or Series")
+        raise TypeError("Input must be a pandas DataFrame or Series.")
 
-    if (npartitions is None) == (chunksize is None):
+    if (npartitions is None) == (none_chunksize := (chunksize is None)):
         raise ValueError("Exactly one of npartitions and chunksize must be specified.")
 
     nrows = len(data)
 
-    if chunksize is None:
+    if none_chunksize:
+        if not isinstance(npartitions, int):
+            raise TypeError(
+                "Please provide npartitions as an int, or possibly as None if you specify chunksize."
+            )
         chunksize = int(ceil(nrows / npartitions))
+    elif not isinstance(chunksize, int):
+        raise TypeError(
+            "Please provide chunksize as an int, or possibly as None if you specify npartitions."
+        )
 
     name = name or ("from_pandas-" + tokenize(data, chunksize))
 
     if not nrows:
         return new_dd_object({(name, 0): data}, name, data, [None, None])
 
-    if sort and not data.index.is_monotonic_increasing:
-        data = data.sort_index(ascending=True)
+    if data.index.isna().any() and not data.index.is_numeric():
+        raise NotImplementedError(
+            "Index in passed data is non-numeric and contains nulls, which Dask does not entirely support.\n"
+            "Consider passing `data.loc[~data.isna()]` instead."
+        )
+
     if sort:
+        if not data.index.is_monotonic_increasing:
+            data = data.sort_index(ascending=True)
         divisions, locations = sorted_division_locations(
             data.index, chunksize=chunksize
         )
@@ -750,5 +776,302 @@ def sorted_division_locations(seq, npartitions=None, chunksize=None):
     return values, positions
 
 
+class _PackedArgCallable(DataFrameIOFunction):
+    """Packed-argument wrapper for DataFrameIOFunction
+
+    This is a private helper class for ``from_map``. This class
+    ensures that packed positional arguments will be expanded
+    before the underlying function (``func``) is called. This class
+    also handles optional metadata enforcement and column projection
+    (when ``func`` satisfies the ``DataFrameIOFunction`` protocol).
+    """
+
+    def __init__(
+        self,
+        func,
+        args=None,
+        kwargs=None,
+        meta=None,
+        packed=False,
+        enforce_metadata=False,
+    ):
+        self.func = func
+        self.args = args
+        self.kwargs = kwargs
+        self.meta = meta
+        self.enforce_metadata = enforce_metadata
+        self.packed = packed
+        self.is_dataframe_io_func = isinstance(self.func, DataFrameIOFunction)
+
+    @property
+    def columns(self):
+        if self.is_dataframe_io_func:
+            return self.func.columns
+        return None
+
+    def project_columns(self, columns):
+        if self.is_dataframe_io_func:
+            return _PackedArgCallable(
+                self.func.project_columns(columns),
+                args=self.args,
+                kwargs=self.kwargs,
+                meta=self.meta[columns],
+                packed=self.packed,
+                enforce_metadata=self.enforce_metadata,
+            )
+        return self
+
+    def __call__(self, packed_arg):
+        if not self.packed:
+            packed_arg = [packed_arg]
+        if self.enforce_metadata:
+            return apply_and_enforce(
+                *packed_arg,
+                *(self.args or []),
+                _func=self.func,
+                _meta=self.meta,
+                **(self.kwargs or {}),
+            )
+        return self.func(
+            *packed_arg,
+            *(self.args or []),
+            **(self.kwargs or {}),
+        )
+
+
+@insert_meta_param_description
+def from_map(
+    func,
+    *iterables,
+    args=None,
+    meta=None,
+    divisions=None,
+    label=None,
+    token=None,
+    enforce_metadata=True,
+    **kwargs,
+):
+    """Create a DataFrame collection from a custom function map
+
+    WARNING: The ``from_map`` API is experimental, and stability is not
+    yet guaranteed. Use at your own risk!
+
+    Parameters
+    ----------
+    func : callable
+        Function used to create each partition. If ``func`` satisfies the
+        ``DataFrameIOFunction`` protocol, column projection will be enabled.
+    *iterables : Iterable objects
+        Iterable objects to map to each output partition. All iterables must
+        be the same length. This length determines the number of partitions
+        in the output collection (only one element of each iterable will
+        be passed to ``func`` for each partition).
+    args : list or tuple, optional
+        Positional arguments to broadcast to each output partition. Note
+        that these arguments will always be passed to ``func`` after the
+        ``iterables`` positional arguments.
+    $META
+    divisions : tuple, str, optional
+        Partition boundaries along the index.
+        For tuple, see https://docs.dask.org/en/latest/dataframe-design.html#partitions
+        For string 'sorted' will compute the delayed values to find index
+        values.  Assumes that the indexes are mutually sorted.
+        If None, then won't use index information
+    label : str, optional
+        String to use as the function-name label in the output
+        collection-key names.
+    token : str, optional
+        String to use as the "token" in the output collection-key names.
+    enforce_metadata : bool, default True
+        Whether to enforce at runtime that the structure of the DataFrame
+        produced by ``func`` actually matches the structure of ``meta``.
+        This will rename and reorder columns for each partition,
+        and will raise an error if this doesn't work or types don't match.
+    **kwargs:
+        Key-word arguments to broadcast to each output partition. These
+        same arguments will be passed to ``func`` for every output partition.
+
+    Examples
+    --------
+    >>> import pandas as pd
+    >>> import dask.dataframe as dd
+    >>> func = lambda x, size=0: pd.Series([x] * size)
+    >>> inputs = ["A", "B"]
+    >>> dd.from_map(func, inputs, size=2).compute()
+    0    A
+    1    A
+    0    B
+    1    B
+    dtype: object
+
+    This API can also be used as an alternative to other file-based
+    IO functions, like ``read_parquet`` (which are already just
+    ``from_map`` wrapper functions):
+
+    >>> import pandas as pd
+    >>> import dask.dataframe as dd
+    >>> paths = ["0.parquet", "1.parquet", "2.parquet"]
+    >>> dd.from_map(pd.read_parquet, paths).head()  # doctest: +SKIP
+                        name
+    timestamp
+    2000-01-01 00:00:00   Laura
+    2000-01-01 00:00:01  Oliver
+    2000-01-01 00:00:02   Alice
+    2000-01-01 00:00:03  Victor
+    2000-01-01 00:00:04     Bob
+
+    Since ``from_map`` allows you to map an arbitrary function
+    to any number of iterable objects, it can be a very convenient
+    means of implementing functionality that may be missing from
+    from other DataFrame-creation methods. For example, if you
+    happen to have apriori knowledge about the number of rows
+    in each of the files in a dataset, you can generate a
+    DataFrame collection with a global RangeIndex:
+
+    >>> import pandas as pd
+    >>> import numpy as np
+    >>> import dask.dataframe as dd
+    >>> paths = ["0.parquet", "1.parquet", "2.parquet"]
+    >>> file_sizes = [86400, 86400, 86400]
+    >>> def func(path, row_offset):
+    ...     # Read parquet file and set RangeIndex offset
+    ...     df = pd.read_parquet(path)
+    ...     return df.set_index(
+    ...         pd.RangeIndex(row_offset, row_offset+len(df))
+    ...     )
+    >>> def get_ddf(paths, file_sizes):
+    ...     offsets = [0] + list(np.cumsum(file_sizes))
+    ...     return dd.from_map(
+    ...         func, paths, offsets[:-1], divisions=offsets
+    ...     )
+    >>> ddf = get_ddf(paths, file_sizes)  # doctest: +SKIP
+    >>> ddf.index  # doctest: +SKIP
+    Dask Index Structure:
+    npartitions=3
+    0         int64
+    86400       ...
+    172800      ...
+    259200      ...
+    dtype: int64
+    Dask Name: myfunc, 6 tasks
+
+    See Also
+    --------
+    dask.dataframe.from_delayed
+    dask.layers.DataFrameIOLayer
+    """
+
+    # Input validation
+    if not callable(func):
+        raise ValueError("`func` argument must be `callable`")
+    lengths = set()
+    iterables = list(iterables)
+    for i, iterable in enumerate(iterables):
+        if not isinstance(iterable, Iterable):
+            raise ValueError(
+                f"All elements of `iterables` must be Iterable, got {type(iterable)}"
+            )
+        try:
+            lengths.add(len(iterable))
+        except (AttributeError, TypeError):
+            iterables[i] = list(iterable)
+            lengths.add(len(iterables[i]))
+    if len(lengths) == 0:
+        raise ValueError("`from_map` requires at least one Iterable input")
+    elif len(lengths) > 1:
+        raise ValueError("All `iterables` must have the same length")
+    if lengths == {0}:
+        raise ValueError("All `iterables` must have a non-zero length")
+
+    # Check for `produces_tasks` and `creation_info`.
+    # These options are included in the function signature,
+    # because they are not intended for "public" use.
+    produces_tasks = kwargs.pop("produces_tasks", False)
+    creation_info = kwargs.pop("creation_info", None)
+
+    if produces_tasks or len(iterables) == 1:
+        if len(iterables) > 1:
+            # Tasks are not detected correctly when they are "packed"
+            # within an outer list/tuple
+            raise ValueError(
+                "Multiple iterables not supported when produces_tasks=True"
+            )
+        inputs = iterables[0]
+        packed = False
+    else:
+        inputs = list(zip(*iterables))
+        packed = True
+
+    # Define collection name
+    label = label or funcname(func)
+    token = token or tokenize(
+        func, meta, inputs, args, divisions, enforce_metadata, **kwargs
+    )
+    name = f"{label}-{token}"
+
+    # Get "projectable" column selection.
+    # Note that this relies on the IO function
+    # ducktyping with DataFrameIOFunction
+    column_projection = func.columns if isinstance(func, DataFrameIOFunction) else None
+
+    # NOTE: Most of the metadata-handling logic used here
+    # is copied directly from `map_partitions`
+    if meta is None:
+        meta = _emulate(
+            func,
+            *(inputs[0] if packed else inputs[:1]),
+            *(args or []),
+            udf=True,
+            **kwargs,
+        )
+        meta_is_emulated = True
+    else:
+        meta = make_meta(meta)
+        meta_is_emulated = False
+
+    if not (has_parallel_type(meta) or is_arraylike(meta) and meta.shape):
+        if not meta_is_emulated:
+            raise TypeError(
+                "Meta is not valid, `from_map` expects output to be a pandas object. "
+                "Try passing a pandas object as meta or a dict or tuple representing the "
+                "(name, dtype) of the columns."
+            )
+        # If `meta` is not a pandas object, the concatenated results will be a
+        # different type
+        meta = make_meta(_concat([meta]))
+
+    # Ensure meta is empty DataFrame
+    meta = make_meta(meta)
+
+    # Define io_func
+    if packed or args or kwargs or enforce_metadata:
+        io_func = _PackedArgCallable(
+            func,
+            args=args,
+            kwargs=kwargs,
+            meta=meta if enforce_metadata else None,
+            enforce_metadata=enforce_metadata,
+            packed=packed,
+        )
+    else:
+        io_func = func
+
+    # Construct DataFrameIOLayer
+    layer = DataFrameIOLayer(
+        name,
+        column_projection,
+        inputs,
+        io_func,
+        label=label,
+        produces_tasks=produces_tasks,
+        creation_info=creation_info,
+    )
+
+    # Return new DataFrame-collection object
+    divisions = divisions or [None] * (len(inputs) + 1)
+    graph = HighLevelGraph.from_collections(name, layer, dependencies=[])
+    return new_dd_object(graph, name, meta, divisions)
+
+
 DataFrame.to_records.__doc__ = to_records.__doc__
 DataFrame.to_bag.__doc__ = to_bag.__doc__
